#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "utility.h"

#define USAGE "USAGE: mask seqfile mask_seqfile\n"

#define NUM_NUC 4     /* Size of nucleotide alphabet */

#define MAX_NAME 256

typedef struct {
  char* seq;
  char name[MAX_NAME];
  double time;
} Sequence;

typedef struct {
  int n_seq;
  int seq_length;
  Sequence* sequence;
  Sequence* mask;
} Params;

char NAUC[7] = "TCAGN-";

Sequence* read_sequence(int* n_seq, int* seq_length, char* seqfile);
void read_params(int argc, char* argv[], Params* params);
void print_sequences(Params* params);

int main(int argc, char* argv[]) {
  Params *params;
  int i, j;

  (params = (Params*)malloc(sizeof(Params))) || die("Could not allocate memory for params\n");
  read_params(argc, argv, params);

  for(i = 0; i < params->n_seq; i++)
    for(j = 0; j < params->seq_length; j++)
      if(params->mask[i].seq[j] >= NUM_NUC)
	params->sequence[i].seq[j] = params->mask[i].seq[j];
  print_sequences(params);
}

void print_sequences(Params* params) {
  int i, j;

  printf("%d %d\n", params->n_seq, params->seq_length);

  for(i = 0; i < params->n_seq; i++) {
    printf("%-10s", params->sequence[i].name);
    for(j = 0; j < params->seq_length; j++)
      printf("%c", NAUC[params->sequence[i].seq[j]]);
    printf("\n");
  }
}


void read_params(int argc, char* argv[], Params* params) {
  int i;
  int n_seq, seq_length;

  for(i = 1; i < argc && argv[i][0] == '-'; i++) {

    die(USAGE);
  }
  argc == i + 2 || die(USAGE);
  params->sequence = read_sequence(&params->n_seq, &params->seq_length, argv[i++]);
  params->mask = read_sequence(&n_seq, &seq_length, argv[i]);
  n_seq == params->n_seq || die("Mismatch in number of sequences\n");
  seq_length == params->seq_length || die("Mismatch in sequence lengths\n");
}

Sequence* read_sequence(int* n_seq, int* seq_length, char* seqfile) {
  FILE* in;
  int i, j, ch;
  char* cp;
  Sequence* sequence;

  (in = fopen(seqfile, "r")) || die("Could not open seqfile: %s\n", seqfile);
  fscanf(in, "%d%d\n", n_seq, seq_length) == 2 || die("Error in seqfile: %s\n", seqfile);
  (sequence = (Sequence*)malloc(sizeof(Sequence) * *n_seq)) || die("Could not allocate memory for sequence\n");
  for(i = 0; i < *n_seq; i++) {
    fscanf(in, "%s.63", sequence[i].name) == 1 || die("Error in seqfile\n");
    (sequence[i].seq = (char*)malloc(sizeof(char) * *seq_length)) || die("Could not allocate memory for sequence[%d].seq\n", i);
    for(j = 0; j < *seq_length;) {
      if((ch = getc(in)) != EOF && (cp = strchr(NAUC, ch)))
	sequence[i].seq[j++] = (unsigned int)cp - (unsigned int)NAUC;
      else if(ch == '\n' || ch == EOF)
	die("Early sequence termination in sequence: %s, %d expected, %d found\n", sequence[i].name, *seq_length, j);
    }
    for(; ch != '\n' && ch != EOF; ch = getc(in));
  }

  fclose(in);
  return sequence;
}
